package gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

import org.opencv.core.Mat;

import dataManagement.DataManager;
import dataManagement.Page;
import gui.initParameters.PnlPages;
import gui.initParameters.PnlProcessParameters;
import gui.initParameters.PnlRegionParameters;
import net.miginfocom.swing.MigLayout;
import segmentation.ImageProcessor;

@SuppressWarnings("serial")
public class Gui extends JFrame {

//	private static final String LAYOUT_DEBUG = "debug";
	 private static final String LAYOUT_DEBUG = "";

	private int verticalResolution;

	private GuiManager guiManager;
	private DataManager dataManager;
	
	private BackgroundPanel pnlBackground;
	private JPanel pnlImage;
	private JPanel pnlControls;
	private JPanel pnlRadioButtons;
	private PnlPages pnlPages;
	
	private PnlRegionParameters pnlRegionParameters;
	private PnlProcessParameters pnlProcessParameters;
	
	private JRadioButton rbOriginal;
	private JRadioButton rbBinary;
	private JRadioButton rbResult;
	
	private JButton btnDoSomething;
	private JButton btnSegmentAll;
	private JButton btnPrevious;
	private JButton btnNext;

	public Gui(ArrayList<Page> pages, String outputFolder, int verticalResolution) {
		super();
		
		setVerticalResolution(verticalResolution);
		pages.get(0).initPage(verticalResolution, null);
		setDataManager(new DataManager(pages));
		setGuiManager(new GuiManager(this));
		guiManager.setOutputFolder(outputFolder);
		
		createWidgets(pages.get(0).getResized());
		addWidgets();
		addListener();

		guiManager.getResizableManager().setPnlBackground(pnlBackground);
		guiManager.getResizableManager().setLastSize(pages.get(0).getResized().size());
		
		pack();
		setLocationRelativeTo(null);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setResizable(false);
	}

	public void showImage(Mat image) {
		Mat resized = ImageProcessor.resize(image, verticalResolution);
		pnlBackground.updateImage(resized);
	}
	
	private void addListener() {		
		btnNext.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				guiManager.showNextPage();
			}
		});
		
		btnPrevious.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				guiManager.showPreviousPage();
			}
		});
		
		btnDoSomething.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				guiManager.segmentCurrentPage();
				rbBinary.setEnabled(true);
				rbResult.setEnabled(true);
				rbResult.setSelected(true);
				rbBinary.setSelected(false);
				rbOriginal.setSelected(false);
			}
		});
		
		btnSegmentAll.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if(btnSegmentAll.getText().equals("Segment All")) {
					btnSegmentAll.setText("Stop");
					guiManager.segmentAllPages();
//					rbBinary.setEnabled(true);
//					rbResult.setEnabled(true);
//					rbResult.setSelected(true);
//					rbBinary.setSelected(false);
//					rbOriginal.setSelected(false);
				} else {
					btnSegmentAll.setText("Segment All");
					guiManager.setKeepSegmenting(false);
				}
			}
		});
		
		rbOriginal.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				rbOriginal.setSelected(true);
				rbBinary.setSelected(false);
				rbResult.setSelected(false);
				
				showImage(dataManager.getCurrentPage().getResized());
			}
		});
		
		rbBinary.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				rbOriginal.setSelected(false);
				rbBinary.setSelected(true);
				rbResult.setSelected(false);
				
				guiManager.calcAndShowBinary();
			}
		});
		
		rbResult.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				rbOriginal.setSelected(false);
				rbBinary.setSelected(false);
				rbResult.setSelected(true);
				
				showImage(dataManager.getCurrentPage().getResult());
			}
		});
	}

	private void addWidgets() {
		MigLayout layout = new MigLayout(LAYOUT_DEBUG);
		setLayout(layout);
		add(pnlImage);
		add(pnlControls);
		
		MigLayout layoutImage = new MigLayout(LAYOUT_DEBUG);
		pnlImage.setLayout(layoutImage);
		
		MigLayout layoutButtons = new MigLayout(LAYOUT_DEBUG);
		pnlRadioButtons.setLayout(layoutButtons);
		pnlRadioButtons.add(btnPrevious, "sg 1");
		pnlRadioButtons.add(btnNext, "sg 1");
		pnlRadioButtons.add(rbOriginal);
		pnlRadioButtons.add(rbBinary);
		pnlRadioButtons.add(rbResult);
		
		pnlImage.add(pnlBackground, "wrap");
		pnlImage.add(pnlRadioButtons, "center");
		
		
		MigLayout layoutControls = new MigLayout(LAYOUT_DEBUG);
		pnlControls.setLayout(layoutControls);
		
		pnlControls.add(pnlRegionParameters, "sgx 1, wrap");
		pnlControls.add(pnlProcessParameters, "sgx 1, wrap");
		pnlControls.add(pnlPages, "sgx 1, wrap");
		pnlControls.add(btnDoSomething, "sg 2, wrap");
		pnlControls.add(btnSegmentAll, "sg 2, wrap");
		
//		pnlControls.add(pnlRegionParameters);
//		pnlControls.add(pnlProcessParameters, "wrap");
//		pnlControls.add(pnlPages, "sgx 1, wrap");
//		pnlControls.add(btnDoSomething, "sg 2, wrap");
//		pnlControls.add(btnSegmentAll, "sg 2, wrap");
	}

	private void createWidgets(Mat input) {
		setContentPane(new JPanel());

		pnlBackground = new BackgroundPanel(input, guiManager.getResizableManager());

		pnlImage = new JPanel();
		pnlControls = new JPanel();
		pnlPages = new PnlPages(guiManager, LAYOUT_DEBUG);
		pnlRadioButtons = new JPanel();
		
		pnlRegionParameters = new PnlRegionParameters(this, LAYOUT_DEBUG);
		pnlProcessParameters = new PnlProcessParameters(guiManager, LAYOUT_DEBUG);
		
		rbOriginal = new JRadioButton("Original");
		rbOriginal.setSelected(true);
		rbBinary = new JRadioButton("Binary");
		rbBinary.setEnabled(true);
		rbResult = new JRadioButton("Result");
		rbResult.setEnabled(false);
		
		btnPrevious = new JButton("Previous");
		btnNext = new JButton("Next");
		btnDoSomething = new JButton("Segment");
		btnSegmentAll = new JButton("Segment All");
	}
	
	public int getVerticalResolution() {
		return verticalResolution;
	}

	public void setVerticalResolution(int verticalResolution) {
		this.verticalResolution = verticalResolution;
	}

	public GuiManager getGuiManager() {
		return guiManager;
	}

	public void setGuiManager(GuiManager guiManager) {
		this.guiManager = guiManager;
	}

	public DataManager getDataManager() {
		return dataManager;
	}

	public void setDataManager(DataManager dataManager) {
		this.dataManager = dataManager;
	}

	public PnlRegionParameters getPnlRegionParameters() {
		return pnlRegionParameters;
	}

	public void setPnlRegionParameters(PnlRegionParameters pnlRegionParameters) {
		this.pnlRegionParameters = pnlRegionParameters;
	}

	public PnlProcessParameters getPnlProcessParameters() {
		return pnlProcessParameters;
	}

	public void setPnlProcessParameters(PnlProcessParameters pnlProcessParameters) {
		this.pnlProcessParameters = pnlProcessParameters;
	}

	public PnlPages getPnlPages() {
		return pnlPages;
	}

	public void setPnlPages(PnlPages pnlPages) {
		this.pnlPages = pnlPages;
	}

	public JRadioButton getRbOriginal() {
		return rbOriginal;
	}

	public void setRbOriginal(JRadioButton rbOriginal) {
		this.rbOriginal = rbOriginal;
	}

	public JRadioButton getRbBinary() {
		return rbBinary;
	}

	public void setRbBinary(JRadioButton rbBinary) {
		this.rbBinary = rbBinary;
	}

	public JRadioButton getRbResult() {
		return rbResult;
	}

	public void setRbResult(JRadioButton rbResult) {
		this.rbResult = rbResult;
	}

	public BackgroundPanel getPnlBackground() {
		return pnlBackground;
	}

	public void setPnlBackground(BackgroundPanel pnlBackground) {
		this.pnlBackground = pnlBackground;
	}	
}