package gui.initParameters;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.font.TextAttribute;
import java.io.File;
import java.util.ArrayList;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.DefaultListCellRenderer;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.UIDefaults;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import dataManagement.Page;
import gui.GuiManager;
import net.miginfocom.swing.MigLayout;

@SuppressWarnings("serial")
public class PnlPages extends JPanel {

	private String layoutDebug;

	private JButton btnAccept;

	private JCheckBox cbShowAccepted;

	private JList<?> listPages;
	private JScrollPane scrollList;

	@SuppressWarnings("rawtypes")
	private DefaultListModel modelList;

	public ArrayList<Integer> processedPages;
	public ArrayList<Integer> acceptedPages;

	private Font fontAccepted;

	public PnlPages(GuiManager guiManager, String layoutDebug) {
		super();

		setLayoutDebug(layoutDebug);

		createWidgets(guiManager.getDataManager().getPages());
		addWidgets();
		addListener(guiManager);

		processedPages = new ArrayList<Integer>();
		acceptedPages = new ArrayList<Integer>();

		initFontAccepted();
	}

	private void addListener(final GuiManager guiManager) {
		btnAccept.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if (btnAccept.getText().equals("Accept")) {
					guiManager.saveCurrentPage();

					int index = listPages.getSelectedIndex();

					Page page = guiManager.getDataManager().getPages().get(index);
					page.setAccepted(true);

					acceptedPages.add(index);

					guiManager.showNextPage();
				} else if (btnAccept.getText().equals("Restore")) {
					int index = listPages.getSelectedIndex();

					Page page = guiManager.getDataManager().getPages().get(index);
					page.setAccepted(false);

					for (Integer i : acceptedPages) {
						String temp = modelList.getElementAt(i).toString();
						if (temp.equals(page.getFileName())) {
							acceptedPages.remove(i);

							String outputFolder = guiManager.getOutputFolder();

							if (!outputFolder.endsWith(File.separator)) {
								outputFolder += File.separator;
							}

							String outputPath = outputFolder + page.getFileName() + ".xml";
							File file = new File(outputPath);
							file.delete();							

							break;
						}
					}



					listPages.repaint();
				}

				// if (index > -1) {
				// modelList.removeElement(modelList.getElementAt(index));
				//
				// listPages.repaint();
				//
				// if (index < modelList.size()) {
				// if (index < modelList.size() && modelList.getElementAt(index)
				// != null) {
				// listPages.setSelectedIndex(index);
				// }
				// } else {
				// listPages.setSelectedIndex(modelList.size() - 1);
				// }
				// }
			}
		});

		cbShowAccepted.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				listPages.repaint();
			}
		});

		listPages.addListSelectionListener(new ListSelectionListener() {

			@Override
			public void valueChanged(ListSelectionEvent e) {
				if (!e.getValueIsAdjusting()) {
					if (listPages.getSelectedIndex() > -1) {
						Page page = guiManager.identifyPage(listPages.getSelectedValue().toString());
						guiManager.loadPage(page);

						if (page.isAccepted()) {
							btnAccept.setText("Restore");
						} else {
							btnAccept.setText("Accept");
						}
					}
				}
			}
		});
	}

	private void addWidgets() {
		MigLayout layout = new MigLayout(layoutDebug);
		setLayout(layout);

		add(scrollList, "growy, pushy, sgx 1, wrap");
		add(btnAccept, "sg 1, wrap");
		add(cbShowAccepted, "sg 1");
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	private void createWidgets(ArrayList<Page> pages) {
		setBorder(BorderFactory.createTitledBorder("Pages"));
		String[] pageNames = new String[pages.size()];

		for (int i = 0; i < pages.size(); i++) {
			pageNames[i] = pages.get(i).getIdentifier();
		}

		// listPages = new JList<Object>(pageNames);
		listPages = new JList<Object>();
		modelList = new DefaultListModel<>();
		listPages.setModel(modelList);

		for (int i = 0; i < pageNames.length; i++) {
			modelList.addElement(pageNames[i]);
		}

		scrollList = new JScrollPane(listPages);
		listPages.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		listPages.setSelectedIndex(0);

		listPages.setCellRenderer(new DefaultListCellRenderer() {

			@Override
			public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected,
					boolean cellHasFocus) {
				Component c = super.getListCellRendererComponent(list, value, index, cellHasFocus, cellHasFocus);

				UIDefaults defaults = javax.swing.UIManager.getDefaults();

				if (processedPages.contains(index)) {
					setBackground(Color.YELLOW);
				} else {
					setBackground(Color.WHITE);
				}

				if (acceptedPages.contains(index)) {
					if (cbShowAccepted.isSelected()) {
						setBackground(Color.GREEN);
						setFont(fontAccepted);
					} else {
						setFont(defaults.getFont("List.selectionBackground"));
					}
				}

				if (listPages.isSelectedIndex(index)) {
					setBackground(defaults.getColor("List.selectionBackground"));
				}

				return c;
			}
		});

		btnAccept = new JButton("Accept");

		cbShowAccepted = new JCheckBox("Show Accepted");
		cbShowAccepted.setSelected(true);
		cbShowAccepted.setEnabled(false);
	}

	@SuppressWarnings("unchecked")
	private void initFontAccepted() {
		Font font = new Font("helvetica", Font.PLAIN, 12);
		@SuppressWarnings("rawtypes")
		Map attributes = font.getAttributes();
		attributes.put(TextAttribute.STRIKETHROUGH, TextAttribute.STRIKETHROUGH_ON);
		Font newFont = new Font(attributes);

		setFontAccepted(newFont);
	}

	public String getLayoutDebug() {
		return layoutDebug;
	}

	public void setLayoutDebug(String layoutDebug) {
		this.layoutDebug = layoutDebug;
	}

	public JList<?> getListPages() {
		return listPages;
	}

	public void setListPages(JList<?> listPages) {
		this.listPages = listPages;
	}

	public ArrayList<Integer> getProcessedPages() {
		return processedPages;
	}

	public void setProcessedPages(ArrayList<Integer> processedPages) {
		this.processedPages = processedPages;
	}

	@SuppressWarnings("rawtypes")
	public DefaultListModel getModelList() {
		return modelList;
	}

	@SuppressWarnings("rawtypes")
	public void setModelList(DefaultListModel modelList) {
		this.modelList = modelList;
	}

	public Font getFontAccepted() {
		return fontAccepted;
	}

	public void setFontAccepted(Font fontAccepted) {
		this.fontAccepted = fontAccepted;
	}
}