package gui.initParameters;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.text.DefaultFormatter;

import gui.GuiManager;
import net.miginfocom.swing.MigLayout;
import segmentation.parameters.DEFAULT_Parameters;

@SuppressWarnings("serial")
public class PnlProcessParameters extends JPanel {

	private String layoutDebug;

	private JLabel lblImageSize;
	private JLabel lblBinaryThresh;
	private JLabel lblImageDilationX;
	private JLabel lblImageDilationY;
	private JLabel lblTextDilationX;
	private JLabel lblTextDilationY;

	private JSpinner spinImageSize;
	private JSpinner spinBinaryThresh;
	private JSpinner spinImageDilationX;
	private JSpinner spinImageDilationY;
	private JSpinner spinTextDilationX;
	private JSpinner spinTextDilationY;

	public PnlProcessParameters(GuiManager guiManager, String layoutDebug) {
		super();

		setLayoutDebug(layoutDebug);

		createWidgets();
		addWidgets();
		addListener(guiManager);
	}

	private void addListener(final GuiManager guiManager) {
		spinBinaryThresh.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				guiManager.calcAndShowBinary();
			}
		});
	}

	private void addWidgets() {
		MigLayout layout = new MigLayout(layoutDebug);
		setLayout(layout);

		add(lblImageSize, "sg 1");
		add(spinImageSize, "sg 2");
		add(lblBinaryThresh, "sg 3");
		add(spinBinaryThresh, "sg 4, wrap");
		add(lblImageDilationX, "sg 1");
		add(spinImageDilationX, "sg 2");
		add(lblTextDilationX, "sg 3");
		add(spinTextDilationX, "sg 4, wrap");
		add(lblImageDilationY, "sg 1");
		add(spinImageDilationY, "sg 2");
		add(lblTextDilationY, "sg 3");
		add(spinTextDilationY, "sg 4, wrap");
//		add(lblImageSize, "sg 1");
//		add(spinImageSize, "sg 2, wrap");
//		add(lblBinaryThresh, "sg 1");
//		add(spinBinaryThresh, "sg 2, wrap");
//		add(lblImageDilationX, "sg 1");
//		add(spinImageDilationX, "sg 2, wrap");
//		add(lblImageDilationY, "sg 1");
//		add(spinImageDilationY, "sg 2, wrap");
//		add(lblTextDilationX, "sg 1");
//		add(spinTextDilationX, "sg 2, wrap");
//		add(lblTextDilationY, "sg 1");
//		add(spinTextDilationY, "sg 2, wrap");
	}

	private void createWidgets() {
		setBorder(BorderFactory.createTitledBorder("Processing Parameters"));

		lblImageSize = new JLabel("Vertical Resolution");
		lblBinaryThresh = new JLabel("Binary Thresh");
		lblImageDilationX = new JLabel("Image Dilation X");
		lblImageDilationY = new JLabel("Image Dilation Y");
		lblTextDilationX = new JLabel("Text Dilation X");
		lblTextDilationY = new JLabel("Text Dilation Y");

		spinImageSize = new JSpinner(new SpinnerNumberModel(DEFAULT_Parameters.getImageHeightDefault(),
				DEFAULT_Parameters.getImageHeightMin(), DEFAULT_Parameters.getImageHeightMax(),
				DEFAULT_Parameters.getImageHeightStepsize()));
		spinBinaryThresh = new JSpinner(new SpinnerNumberModel(DEFAULT_Parameters.getBinaryThreshDefault(),
				DEFAULT_Parameters.getBinaryThreshMin(), DEFAULT_Parameters.getBinaryThreshMax(),
				DEFAULT_Parameters.getBinaryThreshStepsize()));
		spinImageDilationX = new JSpinner(new SpinnerNumberModel(DEFAULT_Parameters.getImageRemovalDilationXDefault(),
				DEFAULT_Parameters.getImageRemovalDilationXMin(), DEFAULT_Parameters.getImageRemovalDilationXMax(),
				DEFAULT_Parameters.getImageRemovalDilationXStepsize()));
		spinImageDilationY = new JSpinner(new SpinnerNumberModel(DEFAULT_Parameters.getImageRemovalDilationYDefault(),
				DEFAULT_Parameters.getImageRemovalDilationYMin(), DEFAULT_Parameters.getImageRemovalDilationYMax(),
				DEFAULT_Parameters.getImageRemovalDilationYStepsize()));
		spinTextDilationX = new JSpinner(new SpinnerNumberModel(DEFAULT_Parameters.getTextRemovalDilationXDefault(),
				DEFAULT_Parameters.getTextRemovalDilationXMin(), DEFAULT_Parameters.getTextRemovalDilationXMax(),
				DEFAULT_Parameters.getTextRemovalDilationXStepsize()));
		spinTextDilationY = new JSpinner(new SpinnerNumberModel(DEFAULT_Parameters.getTextRemovalDilationYDefault(),
				DEFAULT_Parameters.getTextRemovalDilationYMin(), DEFAULT_Parameters.getTextRemovalDilationYMax(),
				DEFAULT_Parameters.getTextRemovalDilationYStepsize()));

		JComponent comp = spinBinaryThresh.getEditor();
		JFormattedTextField field = (JFormattedTextField) comp.getComponent(0);
		DefaultFormatter formatter = (DefaultFormatter) field.getFormatter();
		formatter.setCommitsOnValidEdit(true);
	}

	public String getLayoutDebug() {
		return layoutDebug;
	}

	public void setLayoutDebug(String layoutDebug) {
		this.layoutDebug = layoutDebug;
	}

	public JLabel getLblImageSize() {
		return lblImageSize;
	}

	public JLabel getLblBinaryThresh() {
		return lblBinaryThresh;
	}

	public JLabel getLblImageDilationX() {
		return lblImageDilationX;
	}

	public JLabel getLblImageDilationY() {
		return lblImageDilationY;
	}

	public JLabel getLblTextDilationX() {
		return lblTextDilationX;
	}

	public JLabel getLblTextDilationY() {
		return lblTextDilationY;
	}

	public JSpinner getSpinImageSize() {
		return spinImageSize;
	}

	public JSpinner getSpinBinaryThresh() {
		return spinBinaryThresh;
	}

	public JSpinner getSpinImageDilationX() {
		return spinImageDilationX;
	}

	public JSpinner getSpinImageDilationY() {
		return spinImageDilationY;
	}

	public JSpinner getSpinTextDilationX() {
		return spinTextDilationX;
	}

	public JSpinner getSpinTextDilationY() {
		return spinTextDilationY;
	}
}