package gui.initParameters;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;

import gui.BackgroundPanel;
import gui.Gui;
import helper.TypeConverter;
import net.miginfocom.swing.MigLayout;
import regions.Resizable;
import regions.type.RegionType;
import segmentation.parameters.DEFAULT_Parameters;

@SuppressWarnings("serial")
public class PnlRegionParameters extends JPanel {

	private String layoutDebug;

	private JPanel pnlSize;
	private JPanel pnlShow;
	private JPanel pnlGeneral;
	private JPanel pnlMethod;
	private JPanel pnlRegions;

	private JLabel lblImageMinSize;
	private JLabel lblTextMinSize;
	private JLabel lblMarginaliaMinSize;
	private JLabel lblPageNumberMinSize;

	private JLabel lblMethodImage;
	private JLabel lblMethodText;
	private JLabel lblMethodMarginalia;
	private JLabel lblMethodPageNumber;

	private JSpinner spinImageMinSize;
	private JSpinner spinTextMinSize;
	private JSpinner spinMarginaliaMinSize;
	private JSpinner spinPageNumberMinSize;

	private JCheckBox cbShowSizes;
	private JCheckBox cbMarginaliaPresent;
	private JCheckBox cbPageNumberTop;
	private JCheckBox cbPageNumberBottom;
	private JCheckBox cbCaptionBelow;

	private JCheckBox cbImageShowPosition;
	private JCheckBox cbTextShowPosition;
	private JCheckBox cbMarginaliaShowPosition;
	private JCheckBox cbPageNumberShowPosition;
	private JCheckBox cbIgnoreShowPosition;

	private JCheckBox cbMethodStandardImage;
	private JCheckBox cbMethodStandardParagraph;
	private JCheckBox cbMethodStandardMarginalia;
	private JCheckBox cbMethodStandardPageNumber;

	private JCheckBox cbMethodFixedImage;
	private JCheckBox cbMethodFixedParagraph;
	private JCheckBox cbMethodFixedMarginalia;
	private JCheckBox cbMethodFixedPageNumber;

	private JCheckBox cbFixedRegion;
	private JCheckBox cbPermanentRegion;
	private JCheckBox cbFixedShowAll;

	private JButton btnChange;
	private JButton btnAdd;

	private JComboBox<String> cobSegmentTypes;

	public PnlRegionParameters(Gui gui, String layoutDebug) {
		super();

		setLayoutDebug(layoutDebug);

		createWidgets();
		addWidgets();
		addListener(gui);
	}

	private void addRegion(Gui gui, RegionType type, boolean isFixed, boolean isPermanent) {
		Resizable newRes = gui.getPnlBackground().getActiveResizable();

		if (newRes != null) {
			newRes.setType(type);
			newRes.setFixed(isFixed);
			newRes.setPermanent(isPermanent);
			gui.getPnlBackground().getPositions().add(newRes);
		}
		gui.getPnlBackground().setActiveResizable(null);
		gui.getPnlBackground().repaint();
	}

	private void addListener(final Gui gui) {
		btnAdd.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				RegionType type = TypeConverter.stringToType(cobSegmentTypes.getSelectedItem().toString());
				addRegion(gui, type, cbFixedRegion.isSelected(), cbPermanentRegion.isSelected());
			}
		});

		btnChange.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				RegionType type = TypeConverter.stringToType(cobSegmentTypes.getSelectedItem().toString());
				Resizable activeRes = gui.getPnlBackground().getActiveResizable();

				if (activeRes != null) {
					activeRes.setType(type);
					activeRes.setFixed(cbFixedRegion.isSelected());
					activeRes.setPermanent(cbPermanentRegion.isSelected());
					
					//??
					if(!gui.getGuiManager().getResizableManager().getPositions().contains(activeRes)) {
						gui.getGuiManager().getResizableManager().getPositions().add(activeRes);
						System.out.println("added");
					}
				}
				
				gui.getPnlBackground().repaint();
			}
		});

		cbPageNumberTop.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if (cbPageNumberTop.isSelected()) {
					cbPageNumberBottom.setSelected(false);
				}
			}
		});

		cbPageNumberBottom.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if (cbPageNumberBottom.isSelected()) {
					cbPageNumberTop.setSelected(false);
				}
			}
		});

		cbShowSizes.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				gui.getGuiManager().getResizableManager().showRegionSizes(cbShowSizes.isSelected());
			}
		});

		cbTextShowPosition.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				gui.getGuiManager().getResizableManager().showPositions(RegionType.PARAGRAPH,
						cbTextShowPosition.isSelected());
			}
		});

		cbImageShowPosition.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				gui.getGuiManager().getResizableManager().showPositions(RegionType.IMAGE,
						cbImageShowPosition.isSelected());
			}
		});

		cbMarginaliaShowPosition.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				gui.getGuiManager().getResizableManager().showPositions(RegionType.MARGINALIA,
						cbMarginaliaShowPosition.isSelected());
			}
		});

		cbPageNumberShowPosition.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				gui.getGuiManager().getResizableManager().showPositions(RegionType.PAGE_NUMBER,
						cbPageNumberShowPosition.isSelected());
			}
		});

		cbFixedShowAll.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				gui.getGuiManager().getResizableManager().showFixedPositions(cbFixedShowAll.isSelected());
				
//				for(Resizable res : gui.getGuiManager().getResizableManager().getPositions()) {
//					System.out.println(res.getType() + " " + res.isVisible() + " " + res.isFixed() + " " + res.isPermanent() + " " + res.getRectOCV());
//				}
//				System.out.println("\n");
			}
		});
	}

	private void addWidgets() {
		MigLayout layout = new MigLayout(layoutDebug);
		setLayout(layout);
		add(pnlGeneral, "wrap");
		// add(pnlShow, "wrap");
		add(pnlSize, "wrap");
//		add(pnlMethod, "wrap");
		add(pnlRegions, "wrap");

		MigLayout generalLayout = new MigLayout(layoutDebug);
		pnlGeneral.setLayout(generalLayout);
		pnlGeneral.add(cbMarginaliaPresent);
		pnlGeneral.add(cbPageNumberTop, "wrap");
		pnlGeneral.add(cbCaptionBelow);
		pnlGeneral.add(cbPageNumberBottom);

		MigLayout showLayout = new MigLayout(layoutDebug);
		pnlShow.setLayout(showLayout);
		pnlShow.add(cbFixedShowAll, "wrap");
		pnlShow.add(cbImageShowPosition);
		pnlShow.add(cbMarginaliaShowPosition);
		pnlShow.add(cbPageNumberShowPosition, "wrap");
		pnlShow.add(cbTextShowPosition);
		pnlShow.add(cbIgnoreShowPosition);

		MigLayout sizeLayout = new MigLayout(layoutDebug);
		pnlSize.setLayout(sizeLayout);
		pnlSize.add(lblImageMinSize);
		pnlSize.add(spinImageMinSize);
		pnlSize.add(lblTextMinSize);
		pnlSize.add(spinMarginaliaMinSize, "wrap");
		pnlSize.add(lblMarginaliaMinSize);
		pnlSize.add(spinTextMinSize);
		pnlSize.add(lblPageNumberMinSize);
		pnlSize.add(spinPageNumberMinSize);

		MigLayout methodLayout = new MigLayout(layoutDebug);
		pnlMethod.setLayout(methodLayout);
		pnlMethod.add(lblMethodImage);
		pnlMethod.add(cbMethodStandardImage);
		pnlMethod.add(cbMethodFixedImage, "wrap");
		pnlMethod.add(lblMethodText);
		pnlMethod.add(cbMethodStandardParagraph);
		pnlMethod.add(cbMethodFixedParagraph, "wrap");
		pnlMethod.add(lblMethodMarginalia);
		pnlMethod.add(cbMethodStandardMarginalia);
		pnlMethod.add(cbMethodFixedMarginalia, "wrap");
		pnlMethod.add(lblMethodPageNumber);
		pnlMethod.add(cbMethodStandardPageNumber);
		pnlMethod.add(cbMethodFixedPageNumber);

		MigLayout regionLayout = new MigLayout(layoutDebug);
		pnlRegions.setLayout(regionLayout);
		pnlRegions.add(btnChange);
		pnlRegions.add(cobSegmentTypes);
		pnlRegions.add(cbFixedRegion);
		pnlRegions.add(cbPermanentRegion, "wrap");
//		pnlRegions.add(btnAdd, "wrap");
		pnlRegions.add(pnlShow, "span");
	}

	private void createWidgets() {
		setBorder(BorderFactory.createTitledBorder("Region Parameters"));

		pnlSize = new JPanel();
		pnlSize.setBorder(BorderFactory.createTitledBorder("Minimum Size"));

		pnlShow = new JPanel();
		pnlShow.setBorder(BorderFactory.createTitledBorder("Show"));

		pnlGeneral = new JPanel();
		pnlGeneral.setBorder(BorderFactory.createTitledBorder("General"));

		pnlMethod = new JPanel();
		pnlMethod.setBorder(BorderFactory.createTitledBorder("Method"));

		pnlRegions = new JPanel();
		pnlRegions.setBorder(BorderFactory.createTitledBorder("Regions"));

		lblImageMinSize = new JLabel("Image");
		lblTextMinSize = new JLabel("Paragraph");
		lblMarginaliaMinSize = new JLabel("Marginalia");
		lblPageNumberMinSize = new JLabel("Page Number");

		lblMethodImage = new JLabel("Image");
		lblMethodText = new JLabel("Paragraph");
		lblMethodMarginalia = new JLabel("Marginalia");
		lblMethodPageNumber = new JLabel("Page Number");

		spinImageMinSize = new JSpinner(new SpinnerNumberModel(DEFAULT_Parameters.getImageMinSizeDefault(),
				DEFAULT_Parameters.getImageMinSizeMin(), DEFAULT_Parameters.getImageMinSizeMax(),
				DEFAULT_Parameters.getImageMinSizeStepsize()));
		spinTextMinSize = new JSpinner(new SpinnerNumberModel(DEFAULT_Parameters.getParagraphMinSizeDefault(),
				DEFAULT_Parameters.getParagraphMinSizeMin(), DEFAULT_Parameters.getParagraphMinSizeMax(),
				DEFAULT_Parameters.getParagraphMinSizeStepsize()));
		spinMarginaliaMinSize = new JSpinner(new SpinnerNumberModel(DEFAULT_Parameters.getMarginaliaMinSizeDefault(),
				DEFAULT_Parameters.getMarginaliaMinSizeMin(), DEFAULT_Parameters.getMarginaliaMinSizeMax(),
				DEFAULT_Parameters.getMarginaliaMinSizeStepsize()));
		spinPageNumberMinSize = new JSpinner(new SpinnerNumberModel(DEFAULT_Parameters.getPageNumberMinSizeDefault(),
				DEFAULT_Parameters.getPageNumberMinSizeMin(), DEFAULT_Parameters.getPageNumberMinSizeMax(),
				DEFAULT_Parameters.getPageNumberMinSizeStepsize()));

		cbShowSizes = new JCheckBox("Show Region Sizes");

		cbImageShowPosition = new JCheckBox("Image");
		cbTextShowPosition = new JCheckBox("Paragraph");
		cbMarginaliaShowPosition = new JCheckBox("Marginalia");
		cbPageNumberShowPosition = new JCheckBox("PageNumber");
		cbIgnoreShowPosition = new JCheckBox("Ignore");

		cbMarginaliaPresent = new JCheckBox("Marginalia Present");
		cbPageNumberTop = new JCheckBox("Page Number Top");
		cbPageNumberBottom = new JCheckBox("Page Number Bottom");
		cbCaptionBelow = new JCheckBox("Caption Below");

		cbMethodStandardImage = new JCheckBox("Standard");
		cbMethodStandardParagraph = new JCheckBox("Standard");
		cbMethodStandardMarginalia = new JCheckBox("Standard");
		cbMethodStandardPageNumber = new JCheckBox("Standard");

		cbMethodStandardImage.setSelected(true);
		cbMethodStandardParagraph.setSelected(true);
		cbMethodStandardMarginalia.setSelected(true);
		cbMethodStandardPageNumber.setSelected(true);

		cbMethodFixedImage = new JCheckBox("Fixed");
		cbMethodFixedParagraph = new JCheckBox("Fixed");
		cbMethodFixedMarginalia = new JCheckBox("Fixed");
		cbMethodFixedPageNumber = new JCheckBox("Fixed");

		cbFixedRegion = new JCheckBox("Fixed");
		cbPermanentRegion = new JCheckBox("Permanent");
		cbFixedShowAll = new JCheckBox("All Fixed");

		btnChange = new JButton("Set");
		btnAdd = new JButton("Add");

		// String[] fileFormats = { "Paragraph", "Heading", "Image", "Caption",
		// "Marginalia", "Page-Number", "Ignore" };
		// Color[] colors = { Color.RED, Color.BLUE, Color.GREEN, Color.CYAN,
		// Color.YELLOW, Color.MAGENTA, Color.WHITE };
		String[] fileFormats = { "Paragraph", "Image", "Marginalia", "Page-Number", "Ignore" };

		cobSegmentTypes = new JComboBox<String>(fileFormats) {
			@Override
			public Dimension getMaximumSize() {
				Dimension max = super.getMaximumSize();
				max.height = getPreferredSize().height;
				return max;
			}
		};
	}

	public String getLayoutDebug() {
		return layoutDebug;
	}

	public void setLayoutDebug(String layoutDebug) {
		this.layoutDebug = layoutDebug;
	}

	public JLabel getLblImageMinSize() {
		return lblImageMinSize;
	}

	public JLabel getLblTextMinSize() {
		return lblTextMinSize;
	}

	public JLabel getLblMarginaliaMinSize() {
		return lblMarginaliaMinSize;
	}

	public JLabel getLblPageNumberMinSize() {
		return lblPageNumberMinSize;
	}

	public JSpinner getSpinImageMinSize() {
		return spinImageMinSize;
	}

	public JSpinner getSpinTextMinSize() {
		return spinTextMinSize;
	}

	public JSpinner getSpinMarginaliaMinSize() {
		return spinMarginaliaMinSize;
	}

	public JSpinner getSpinPageNumberMinSize() {
		return spinPageNumberMinSize;
	}

	public JCheckBox getCbShowSizes() {
		return cbShowSizes;
	}

	public JCheckBox getCbImageShowPosition() {
		return cbImageShowPosition;
	}

	public JCheckBox getCbTextShowPosition() {
		return cbTextShowPosition;
	}

	public JCheckBox getCbMarginaliaShowPosition() {
		return cbMarginaliaShowPosition;
	}

	public JCheckBox getCbPageNumberShowPosition() {
		return cbPageNumberShowPosition;
	}

	public JCheckBox getCbMarginaliaPresent() {
		return cbMarginaliaPresent;
	}

	public void setCbMarginaliaPresent(JCheckBox cbMarginaliaPresent) {
		this.cbMarginaliaPresent = cbMarginaliaPresent;
	}

	public JCheckBox getCbPageNumberTop() {
		return cbPageNumberTop;
	}

	public void setCbPageNumberTop(JCheckBox cbPageNumberTop) {
		this.cbPageNumberTop = cbPageNumberTop;
	}

	public JCheckBox getCbPageNumberBottom() {
		return cbPageNumberBottom;
	}

	public void setCbPageNumberBottom(JCheckBox cbPageNumberBottom) {
		this.cbPageNumberBottom = cbPageNumberBottom;
	}

	public JCheckBox getCbCaptionBelow() {
		return cbCaptionBelow;
	}

	public void setCbCaptionBelow(JCheckBox cbCaptionBelow) {
		this.cbCaptionBelow = cbCaptionBelow;
	}
}