package regions;

import java.awt.Color;
import java.awt.geom.Rectangle2D;

import org.opencv.core.Rect;

import regions.type.RegionType;

public class Resizable {

	private static final int ALPHA_STANDARD = 80;
	private static final int ALPHA_ACTIVE = 150;

	private Rectangle2D rect;
	private Rect rectOCV;
	private boolean isActive;
	private boolean isVisible;
	private boolean isFixed;
	private boolean isPermanent;

	private Color color;
	private Color standardColor;
	private Color activeColor;

	private RegionType type;

	public Resizable(Rectangle2D rect, RegionType regionType) {
		setRect(rect);
		setType(regionType);
		// initColors(regionType);

		setActive(false);
		setVisible(false);
	}

	public void initColors(RegionType regionType) {
		if (regionType.equals(RegionType.IMAGE)) {
			setStandardColor(new Color(0, 255, 0, ALPHA_STANDARD));
			setActiveColor(new Color(0, 255, 0, ALPHA_ACTIVE));
		} else if (regionType.equals(RegionType.PARAGRAPH)) {
			setStandardColor(new Color(255, 0, 0, ALPHA_STANDARD));
			setActiveColor(new Color(255, 0, 0, ALPHA_ACTIVE));
		} else if (regionType.equals(RegionType.MARGINALIA)) {
			setStandardColor(new Color(255, 255, 0, ALPHA_STANDARD));
			setActiveColor(new Color(255, 255, 0, ALPHA_ACTIVE));
		} else if (regionType.equals(RegionType.PAGE_NUMBER)) {
			setStandardColor(new Color(255, 0, 255, ALPHA_STANDARD));
			setActiveColor(new Color(255, 0, 255, ALPHA_ACTIVE));
		} else if (regionType.equals(RegionType.IGNORE)) {
//			setStandardColor(new Color(0, 0, 255, ALPHA_STANDARD));
//			setActiveColor(new Color(0, 0, 255, ALPHA_ACTIVE));
			setStandardColor(new Color(255, 255, 255, ALPHA_STANDARD));
			setActiveColor(new Color(255, 255, 255, ALPHA_ACTIVE));
		} else {
			setStandardColor(new Color(255, 0, 0, ALPHA_STANDARD));
			setActiveColor(new Color(255, 0, 0, ALPHA_ACTIVE));
		}

		if (isActive) {
			setColor(activeColor);
		} else {
			setColor(standardColor);
		}
	}

	public Resizable(Rect rect, RegionType regionType) {
		this(new Rectangle2D.Double(rect.x, rect.y, rect.width, rect.height), regionType);
	}

	public Rectangle2D getRect() {
		return rect;
	}

	public void setRect(Rectangle2D rect) {
		this.rect = rect;
		setRectOCV(rect);
	}

	public Rect getRectOCV() {
		return rectOCV;
	}

	public void setRectOCV(Rectangle2D rect) {
		Rect rectOCV = new Rect((int) rect.getX(), (int) rect.getY(), (int) rect.getWidth(), (int) rect.getHeight());
		this.rectOCV = rectOCV;
	}

	public void setRectOCV(Rect rectOCV) {
		this.rectOCV = rectOCV;
	}

	public boolean isActive() {
		return isActive;
	}

	public void setActive(boolean isActive) {
		if (isActive) {
			setColor(activeColor);
		} else {
			setColor(standardColor);
		}

		this.isActive = isActive;
	}

	public Color getColor() {
		return color;
	}

	public void setColor(Color color) {
		this.color = color;
	}

	public Color getStandardColor() {
		return standardColor;
	}

	public void setStandardColor(Color standardColor) {
		this.standardColor = standardColor;
	}

	public Color getActiveColor() {
		return activeColor;
	}

	public void setActiveColor(Color activeColor) {
		this.activeColor = activeColor;
	}

	public boolean isVisible() {
		return isVisible;
	}

	public void setVisible(boolean isVisible) {
		this.isVisible = isVisible;
	}

	public RegionType getType() {
		return type;
	}

	public void setType(RegionType type) {
		this.type = type;
		initColors(type);
	}

	public boolean isFixed() {
		return isFixed;
	}

	public void setFixed(boolean isFixed) {
		this.isFixed = isFixed;
	}

	public boolean isPermanent() {
		return isPermanent;
	}

	public void setPermanent(boolean isPermanent) {
		this.isPermanent = isPermanent;
	}	
}