package segmentation;

import java.util.ArrayList;

import org.opencv.core.Core;
import org.opencv.core.CvType;
import org.opencv.core.Mat;
import org.opencv.core.MatOfPoint;
import org.opencv.core.Rect;
import org.opencv.core.Scalar;
import org.opencv.imgproc.Imgproc;

import imageProcessing.Contour;

public class ImageSegmentation {

	public static ArrayList<MatOfPoint> combineContoursAsRects(ArrayList<MatOfPoint> contours, Mat image) {
		Mat binary = new Mat(image.size(), CvType.CV_8U, new Scalar(0));

		for (MatOfPoint contour : contours) {
			Rect rect = Imgproc.boundingRect(contour);
			Core.rectangle(binary, rect.tl(), rect.br(), new Scalar(255), -1);
		}

		ArrayList<MatOfPoint> results = Contour.findContours(binary);

		return results;
	}

	public static ArrayList<MatOfPoint> detectContours(Mat binary, int minSize, boolean combine) {
		ArrayList<MatOfPoint> contours = Contour.findContours(binary);
		ArrayList<MatOfPoint> results = new ArrayList<MatOfPoint>();

		for (MatOfPoint contour : contours) {
			Rect rect = Imgproc.boundingRect(contour);

			if (rect.area() > minSize) {
				results.add(contour);
			}
		}

		if (combine) {
			ArrayList<MatOfPoint> combinedResults = combineContoursAsRects(results, binary);
			
			return combinedResults;
		}
		
		return results;
	}
}