package segmentation.parameters;

import gui.Gui;
import regions.ResizableManager;

public class Parameters {

	// Processing Parameters
	private int desiredImageHeight;
	private int binaryThresh;

	private int imageRemovalDilationX;
	private int imageRemovalDilationY;

	private int textDilationX;
	private int textDilationY;

	// Region Parameters
	private int minImageSize;
	private int minTextSize;
	private int minMarginaliaSize;
	private int minPageNumberSize;

	private boolean marginaliaPresent;
	private boolean pageNumberTop;
	private boolean pageNumberBottom;
	private boolean captionBelow;

	private ResizableManager resizableManager;

	private double scaleFactor;

	private boolean cutMarginalia;

	public Parameters(ResizableManager resizableManager, Gui gui) {
		setResizableManager(resizableManager);

		setDesiredImageHeight(Integer.parseInt(gui.getPnlProcessParameters().getSpinImageSize().getValue().toString()));
		setBinaryThresh(Integer.parseInt(gui.getPnlProcessParameters().getSpinBinaryThresh().getValue().toString()));

		setImageRemovalDilationX(
				Integer.parseInt(gui.getPnlProcessParameters().getSpinImageDilationX().getValue().toString()));
		setImageRemovalDilationY(
				Integer.parseInt(gui.getPnlProcessParameters().getSpinImageDilationY().getValue().toString()));

		setTextDilationX(Integer.parseInt(gui.getPnlProcessParameters().getSpinTextDilationX().getValue().toString()));
		setTextDilationY(Integer.parseInt(gui.getPnlProcessParameters().getSpinTextDilationY().getValue().toString()));

		setMinImageSize(Integer.parseInt(gui.getPnlRegionParameters().getSpinImageMinSize().getValue().toString()));
		setMinTextSize(Integer.parseInt(gui.getPnlRegionParameters().getSpinTextMinSize().getValue().toString()));
		setMinMarginaliaSize(
				Integer.parseInt(gui.getPnlRegionParameters().getSpinMarginaliaMinSize().getValue().toString()));
		setMinPageNumberSize(
				Integer.parseInt(gui.getPnlRegionParameters().getSpinPageNumberMinSize().getValue().toString()));

		setMarginaliaPresent(gui.getPnlRegionParameters().getCbMarginaliaPresent().isSelected());
		setPageNumberTop(gui.getPnlRegionParameters().getCbPageNumberTop().isSelected());
		setPageNumberBottom(gui.getPnlRegionParameters().getCbPageNumberBottom().isSelected());
		setCaptionBelow(gui.getPnlRegionParameters().getCbCaptionBelow().isSelected());

		setScaleFactor((double) desiredImageHeight / gui.getVerticalResolution());
	}

	public int getDesiredImageHeight() {
		return desiredImageHeight;
	}

	public void setDesiredImageHeight(int desiredImageHeight) {
		this.desiredImageHeight = desiredImageHeight;
	}

	public int getBinaryThresh() {
		return binaryThresh;
	}

	public void setBinaryThresh(int binaryThresh) {
		this.binaryThresh = binaryThresh;
	}

	public int getImageRemovalDilationX() {
		return imageRemovalDilationX;
	}

	public void setImageRemovalDilationX(int imageRemovalDilationX) {
		this.imageRemovalDilationX = imageRemovalDilationX;
	}

	public int getImageRemovalDilationY() {
		return imageRemovalDilationY;
	}

	public void setImageRemovalDilationY(int imageRemovalDilationY) {
		this.imageRemovalDilationY = imageRemovalDilationY;
	}

	public int getTextDilationX() {
		return textDilationX;
	}

	public void setTextDilationX(int textDilationX) {
		this.textDilationX = textDilationX;
	}

	public int getTextDilationY() {
		return textDilationY;
	}

	public void setTextDilationY(int textDilationY) {
		this.textDilationY = textDilationY;
	}

	public int getMinImageSize() {
		return minImageSize;
	}

	public void setMinImageSize(int minImageSize) {
		this.minImageSize = minImageSize;
	}

	public int getMinTextSize() {
		return minTextSize;
	}

	public void setMinTextSize(int minTextSize) {
		this.minTextSize = minTextSize;
	}

	public int getMinMarginaliaSize() {
		return minMarginaliaSize;
	}

	public void setMinMarginaliaSize(int minMarginaliaSize) {
		this.minMarginaliaSize = minMarginaliaSize;
	}

	public int getMinPageNumberSize() {
		return minPageNumberSize;
	}

	public void setMinPageNumberSize(int minPageNumberSize) {
		this.minPageNumberSize = minPageNumberSize;
	}

	public ResizableManager getResizableManager() {
		return resizableManager;
	}

	public void setResizableManager(ResizableManager resizableManager) {
		this.resizableManager = resizableManager;
	}

	public boolean isMarginaliaPresent() {
		return marginaliaPresent;
	}

	public void setMarginaliaPresent(boolean marginaliaPresent) {
		this.marginaliaPresent = marginaliaPresent;
	}

	public boolean isPageNumberTop() {
		return pageNumberTop;
	}

	public void setPageNumberTop(boolean pageNumberTop) {
		this.pageNumberTop = pageNumberTop;
	}

	public boolean isPageNumberBottom() {
		return pageNumberBottom;
	}

	public void setPageNumberBottom(boolean pageNumberBottom) {
		this.pageNumberBottom = pageNumberBottom;
	}

	public boolean isCaptionBelow() {
		return captionBelow;
	}

	public void setCaptionBelow(boolean captionBelow) {
		this.captionBelow = captionBelow;
	}

	public double getScaleFactor() {
		return scaleFactor;
	}

	public void setScaleFactor(double scaleFactor) {
		this.scaleFactor = scaleFactor;
	}

	public boolean isCutMarginalia() {
		return cutMarginalia;
	}

	public void setCutMarginalia(boolean cutMarginalia) {
		this.cutMarginalia = cutMarginalia;
	}
}