package segmentation.verfication;

import java.util.ArrayList;

import org.opencv.core.Mat;
import org.opencv.core.MatOfPoint;
import org.opencv.core.Point;
import org.opencv.core.Rect;
import org.opencv.imgproc.Imgproc;

import segmentation.parameters.Parameters;

public class PageNumberVerifier {

	private static final double PAGE_NUMBER_VERTICAL_POSITION_PERCENTAGE = 0.20;
	
	private Mat binary;
	private boolean pageNumberTop;
	private double verticalPositionPercentage;

	private Rect validPostion;

	public PageNumberVerifier(Mat binary, Parameters parameters) {
		setBinary(binary);
		setPageNumberTop(parameters.isPageNumberTop());
		setVerticalPositionPercentage(PAGE_NUMBER_VERTICAL_POSITION_PERCENTAGE);
	}

	public int identifyTopBottomPageNumber(ArrayList<MatOfPoint> pageNumberContours) {
		int index = -1;

		if (pageNumberTop) {
			int top = Integer.MAX_VALUE;

			for (int i = 0; i < pageNumberContours.size(); i++) {
				Rect rect = Imgproc.boundingRect(pageNumberContours.get(i));
				if (rect.tl().y < top) {
					index = i;
					top = (int) rect.tl().y;
				}
			}
		} else {
			int bottom = 0;

			for (int i = 0; i < pageNumberContours.size(); i++) {
				Rect rect = Imgproc.boundingRect(pageNumberContours.get(i));
				if (rect.br().y > bottom) {
					index = i;
					bottom = (int) rect.br().y;
				}
			}
		}

		return index;
	}

	public void identifyPageNumber(ArrayList<MatOfPoint> textContours, ArrayList<MatOfPoint> pageNumberContours, double minTextArea) {
		int index = identifyTopBottomPageNumber(pageNumberContours);

		if (index != -1) {
			for (int i = 0; i < pageNumberContours.size(); i++) {
				Rect rect = Imgproc.boundingRect(pageNumberContours.get(i));
				double area = rect.area();
				
				if (i != index && area > minTextArea) {
					textContours.add(pageNumberContours.get(i));
				}
			}

			MatOfPoint pageNumber = pageNumberContours.get(index);
			pageNumberContours.removeAll(pageNumberContours);
			pageNumberContours.add(pageNumber);
		}
	}

	public void initPosition() {
		if (pageNumberTop) {
			Rect top = new Rect(new Point(0, 0),
					new Point(binary.width() - 1, verticalPositionPercentage * binary.height()));
			setValidPostion(top);
		} else {
			Rect bottom = new Rect(new Point(0, (1 - verticalPositionPercentage) * binary.height()),
					new Point(binary.width() - 1, binary.height() - 1));
			setValidPostion(bottom);
		}
	}

	public boolean checkPageNumberPosition(Rect rect) {
		if (validPostion == null) {
			initPosition();
		}

		if (rect.br().y <= validPostion.br().y && rect.br().x <= validPostion.br().x
				&& rect.tl().y >= validPostion.tl().y && rect.tl().x >= validPostion.tl().x) {
			return true;
		} else {
			return false;
		}
	}

	public Mat getBinary() {
		return binary;
	}

	public void setBinary(Mat binary) {
		this.binary = binary;
	}

	public boolean isPageNumberTop() {
		return pageNumberTop;
	}

	public void setPageNumberTop(boolean pageNumberTop) {
		this.pageNumberTop = pageNumberTop;
	}

	public double getVerticalPositionPercentage() {
		return verticalPositionPercentage;
	}

	public void setVerticalPositionPercentage(double verticalPositionPercentage) {
		this.verticalPositionPercentage = verticalPositionPercentage;
	}

	public Rect getValidPostion() {
		return validPostion;
	}

	public void setValidPostion(Rect validPostion) {
		this.validPostion = validPostion;
	}
}